import { EntityComponentTypes, world, system, MolangVariableMap } from "@minecraft/server";

import "./nperma/index";


system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    if (player.hasTag("multi_respawn")) {
      const spawnPoint = player.getSpawnPoint()
      player.teleport(spawnPoint, { dimension: spawnPoint.dimension })
      player.removeTag(`multi_respawn`);
    }
}
});

world.beforeEvents.playerInteractWithBlock.subscribe(({player, block}) => {
  if (block?.typeId === 'minecraft:bed') {
    // interaction event;
    system.run(() => {player.runCommand('tag @s add spawnpoint_set')});
  };
});;

// Create scoreboard objectives to store coordinates and dimension
system.run(() => {
  try {
    world.scoreboard.addObjective("checkpoint_x", "Checkpoint X");
    world.scoreboard.addObjective("checkpoint_y", "Checkpoint Y");
    world.scoreboard.addObjective("checkpoint_z", "Checkpoint Z");
    world.scoreboard.addObjective("checkpoint_dim", "Checkpoint Dimension ID");
    world.scoreboard.addObjective("has_checkpoint", "Has Checkpoint");
  } catch {
    // Objectives might already exist
  }
});

// Handle checkpoint creation
system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    if (player.hasTag("create_checkpoint")) {
      // Get player position
      const position = player.location;
      const dimensionId = player.dimension.id;
      
      // Store coordinates in scoreboard
      try {
        const obj_x = world.scoreboard.getObjective("checkpoint_x");
        const obj_y = world.scoreboard.getObjective("checkpoint_y");
        const obj_z = world.scoreboard.getObjective("checkpoint_z");
        const obj_dim = world.scoreboard.getObjective("checkpoint_dim");
        const obj_has = world.scoreboard.getObjective("has_checkpoint");
        
        // Store as integers (multiply by 100 to preserve 2 decimal places)
        obj_x.setScore(player, Math.floor(position.x * 100));
        obj_y.setScore(player, Math.floor(position.y * 100));
        obj_z.setScore(player, Math.floor(position.z * 100));
        
        // Store the dimension ID as a string in player's tags
        player.removeTag(`checkpoint_dim:overworld`);
        player.removeTag(`checkpoint_dim:nether`);
        player.removeTag(`checkpoint_dim:the_end`);
        player.removeTag(`checkpoint_dim:custom`);
        
        if (dimensionId === "minecraft:overworld") {
          player.addTag(`checkpoint_dim:overworld`);
          obj_dim.setScore(player, 0);
        } else if (dimensionId === "minecraft:nether") {
          player.addTag(`checkpoint_dim:nether`);
          obj_dim.setScore(player, 1);
        } else if (dimensionId === "minecraft:the_end") {
          player.addTag(`checkpoint_dim:the_end`);
          obj_dim.setScore(player, 2);
        } else {
          // Handle custom dimensions by storing a code
          player.addTag(`checkpoint_dim:custom`);
          obj_dim.setScore(player, 3); // 3 for custom dimensions
        }
        
        // Mark that player has a checkpoint
        obj_has.setScore(player, 1);
        
        // Notify player
        player.runCommandAsync(`execute as @s run scriptevent dialog:text °(Savepoint reached. All inventories will be kept til a bed is touched.;1;dialogue.txt2)`);
        player.runCommandAsync(`gamerule keepinventory true`);
      } catch (err) {
        console.warn("Error setting checkpoint: " + err);
      }
      
      // Remove the tag
      player.removeTag("create_checkpoint");
    }
    
    // Check if player manually set spawnpoint
    if (player.hasTag("spawnpoint_set")) {
      // Clear checkpoint data
      try {
        const obj_has = world.scoreboard.getObjective("has_checkpoint");
        obj_has.setScore(player, 0);
        
        // Remove dimension tags
        player.removeTag(`checkpoint_dim:overworld`);
        player.removeTag(`checkpoint_dim:nether`);
        player.removeTag(`checkpoint_dim:the_end`);
        player.removeTag(`checkpoint_dim:custom`);
        
        player.runCommandAsync(`execute as @s[tag=spawnpoint_set] run scriptevent dialog:text °(Savepoint destroyed. Inventories will not be kept.;1;dialogue.txt2)`);
        player.runCommandAsync(`gamerule keepinventory false`);
      } catch (err) {
        console.warn("Error clearing checkpoint: " + err);
      }
      
      // Remove the tag
      player.removeTag("spawnpoint_set");
    }
  }
});

// Handle player death and respawn with inventory
world.afterEvents.playerSpawn.subscribe((event) => {
  const player = event.player;
  
  // Check if this is a respawn after death (not initial join)
  if (event.initialSpawn === false) {
    try {
      // Check if player has a checkpoint
      const obj_has = world.scoreboard.getObjective("has_checkpoint");
      const hasCheckpoint = obj_has.getScore(player);
      
      if (hasCheckpoint === 1) {
        // Get saved checkpoint coordinates
        const obj_x = world.scoreboard.getObjective("checkpoint_x");
        const obj_y = world.scoreboard.getObjective("checkpoint_y");
        const obj_z = world.scoreboard.getObjective("checkpoint_z");
        const obj_dim = world.scoreboard.getObjective("checkpoint_dim");
        
        const x = obj_x.getScore(player) / 100;
        const y = obj_y.getScore(player) / 100;
        const z = obj_z.getScore(player) / 100;
        const dimCode = obj_dim.getScore(player);
        
        // Determine dimension from code
        let dimension;
        
        if (player.hasTag("checkpoint_dim:overworld") || dimCode === 0) {
          dimension = world.getDimension("minecraft:overworld");
        } else if (player.hasTag("checkpoint_dim:nether") || dimCode === 1) {
          dimension = world.getDimension("minecraft:nether");
        } else if (player.hasTag("checkpoint_dim:the_end") || dimCode === 2) {
          dimension = world.getDimension("minecraft:the_end");
        } else {
          // For custom dimensions, default to overworld and log error
          dimension = world.getDimension("minecraft:overworld");
          console.warn("Custom dimension detected - defaulting to overworld");
          player.runCommandAsync(`execute as @s run scriptevent dialog:text °(Unknown;1;dialogue.txt2)`);
          player.runCommandAsync(`gamerule keepinventory false`);
        }
        
        // Teleport to checkpoint with a small delay
        system.runTimeout(() => {
          player.teleport(
            { x: x, y: y, z: z },
            { dimension: dimension }
          );
          
          const dimName = dimension.id.replace("minecraft:", "");
          player.runCommandAsync(`execute as @s run scriptevent dialog:text °(You have unfinished business.;1;dialogue.txt2)`);
        }, 10); // Small delay to ensure teleport works properly after respawn
      }
    } catch (err) {
      console.warn("Error during respawn: " + err);
      // Fallback to default behavior if there's an error
      player.runCommandAsync(`execute as @s run scriptevent dialog:text °(This is not right.;1;dialogue.txt2)`);
      player.runCommandAsync(`gamerule keepinventory false`);
    }
  }
});






// --- Configuration Array ---
// Add objects here for each structure you want to generate.
// - name: The exact name of the saved .mcstructure file (without the extension).
// - location: An object { x, y, z } for the target coordinates.
// - radius: How close a player needs to be to trigger generation.
// - dimensionId: "overworld", "nether", or "the_end".
// - generatedProperty: A UNIQUE key for world.setDynamicProperty to track this specific generation.
//                      Best practice is to make it descriptive (e.g., structure name + coords).
const structuresToConfigure = [
    {
        name: "skel_dr_tower",
        location: { x: 74936, y: 159, z: -25316 },
        radius: 40,
        dimensionId: "overworld",
        generatedProperty: "structgen_skel_dr_tower_74936_159_-25316" // Unique key
    },
    {
      name: "algid_wither_shack",
      location: { x: 25170, y: 128, z: -75085 },
      radius: 30,
      dimensionId: "the_end",
      generatedProperty: "structgen_algid_wither_shack_25170_128_-75085" // Unique key
    },
    {
      name: "nuclear_golem_lab",
      location: { x: 74869, y: 44, z: 24997 },
      radius: 35,
      dimensionId: "nether",
      generatedProperty: "structgen_nuclear_golem_lab_74869_44_24997" // Unique key
    },
    {
      name: "pigman_cathedral",
      location: { x: 74980, y: 226, z: 25050 },
      radius: 30,
      dimensionId: "overworld",
      generatedProperty: "structgen_pigman_cathedral_74980_226_25050" // Unique key
    },
    {
      name: "firic_tomb",
      location: { x: 75066, y: 75, z: -24965 },
      radius: 20,
      dimensionId: "nether",
      generatedProperty: "structgen_firic_tomb_75066_75_-24965" // Unique key
    },
    {
      name: "spider_lab",
      location: { x: 75125, y: 86, z: 75025 },
      radius: 25,
      dimensionId: "nether",
      generatedProperty: "structgen_spider_lab_75125_86_75025" // Unique key
    },
    {
      name: "summerset_arena",
      location: { x: 75110, y: 97, z: -75150 },
      radius: 45,
      dimensionId: "the_end",
      generatedProperty: "structgen_algid_summerset_arena_75110_97_-75150" // Unique key
    },
    {
      name: "dd_trojan_horse",
      location: { x: -25125, y: 88, z: -75277 },
      radius: 45,
      dimensionId: "the_end",
      generatedProperty: "structgen_dd_trojan_horse_-25125_93_-75277" // Unique key
    }
    // Add more structure configurations here as needed 
];
// --- Pre-calculate squared radii for efficiency ---
const structuresConfig = structuresToConfigure.map(config => ({
    ...config, // Copy existing properties
    radiusSquared: config.radius * config.radius, // Add the squared radius
    dimension: world.getDimension(config.dimensionId) // Get dimension object
}));

const CHECK_INTERVAL = 20; // How often to check (in game ticks, 20 ticks = 1 second)

console.warn(`[Multi Structure Gen] Script Loaded. Monitoring ${structuresConfig.length} configurations.`);

// --- Main Check Loop ---
// Keep intervalId in case you want to stop checks later (though less useful here as configs are independent)
// eslint-disable-next-line @typescript-eslint/no-unused-vars
const intervalId = system.runInterval(() => {
    const players = world.getAllPlayers();

    // Iterate through each structure configuration
    for (const config of structuresConfig) {
        // --- Check if THIS structure has already been generated ---
        let alreadyGenerated = false;
        try {
            const propertyValue = world.getDynamicProperty(config.generatedProperty);
            if (propertyValue === true) {
                alreadyGenerated = true;
            }
        } catch (_error) {
            alreadyGenerated = false; // Property doesn't exist, so not generated
        }

        // If already generated for this config, skip to the next config
        if (alreadyGenerated) {
            continue;
        }

        // --- If not generated, check players for THIS configuration ---
        const targetDim = config.dimension;
        const targetLoc = config.location;
        const targetRadiusSq = config.radiusSquared;

        for (const player of players) {
            // Check if player is in the correct dimension for this structure config
            if (player.dimension.id !== targetDim.id) {
                continue;
            }

            // Calculate squared distance from player to THIS structure's target location
            const dx = player.location.x - targetLoc.x;
            const dy = player.location.y - targetLoc.y;
            const dz = player.location.z - targetLoc.z;
            const distanceSquared = dx * dx + dy * dy + dz * dz;

            // Check if player is within the trigger radius for THIS config
            if (distanceSquared <= targetRadiusSq) {
                console.warn(`[Multi Structure Gen] Player ${player.name} detected near ${config.name} target (${targetLoc.x}, ${targetLoc.y}, ${targetLoc.z}). Attempting generation...`);

                try {
                    // Execute the structure load command for THIS config
                    targetDim.runCommandAsync(`structure load "${config.name}" ${targetLoc.x} ${targetLoc.y} ${targetLoc.z} 90_degrees`)
                        .then(() => {
                            console.warn(`[Multi Structure Gen] Successfully executed command for structure: ${config.name}`);
                            // Set the UNIQUE dynamic property flag for THIS structure
                            world.setDynamicProperty(config.generatedProperty, true);
                        })
                        .catch((commandError) => {
                            console.error(`[Multi Structure Gen] Failed to execute structure load command for ${config.name}: ${commandError}`);
                            // Consider if you want to set the flag even on failure to prevent command spam
                            // world.setDynamicProperty(config.generatedProperty, true);
                        });

                    // Structure triggered for this config, break from player loop
                    // and move to the next structure configuration check.
                    // No need to check other players for *this same structure* in this tick.
                    break;

                } catch (error) {
                    console.error(`[Multi Structure Gen] Unexpected error during structure load attempt for ${config.name}: ${error}`);
                     // Consider if you want to set the flag even on failure
                    // world.setDynamicProperty(config.generatedProperty, true);
                }
            } // End of distance check
        } // End of player loop (for this config)
    } // End of configuration loop
}, CHECK_INTERVAL); // Run this check function every CHECK_INTERVAL ticks


//new thing


// --- Configuration ---

// List of coordinates to check proximity to.
const targetCoordinates = [
  { id: "skel_dr_tower", x: 74936, y: 159, z: -25316, dimensionId: "minecraft:overworld", name: "Tower of the Last Dragon" },
  { id: "algid_wither_shack", x: 25170, y: 128, z: -75085, dimensionId: "minecraft:the_end", name: "Shack of a dead god" },
  { id: "nuclear_golem_lab", x: 74869, y: 44, z: 24997, dimensionId: "minecraft:nether", name: "Last Chamber of Man" },
  { id: "pigman_cathedral", x: 74980, y: 226, z: 25050, dimensionId: "minecraft:overworld", name: "Hall of an old hope" },
  { id: "firic_tomb", x: 75066, y: 75, z: -24965, dimensionId: "minecraft:nether", name: "Tomb of the Flaming King" },
  { id: "spider_lab", x: 75125, y: 86, z: 75025, dimensionId: "minecraft:nether", name: "Lab of Arthropods" },
  { id: "summerset_arena", x: 75110, y: 97, z: -75150, dimensionId: "minecraft:the_end", name: "The Worthless Finale" },
  { id: "dd_trojan_horse", x: -25125, y: 88, z: -75277, dimensionId: "minecraft:the_end", name: "The Stede of Fools" }

  
  // Add more coordinates here, ensuring each has a unique 'id'
];

const proximityRadius = 512;
const proximityRadiusSquared = proximityRadius * proximityRadius;
const arrivalRadius = 10;
const arrivalRadiusSquared = arrivalRadius * arrivalRadius;
const visitedTagPrefix = "prox_visited_";
const checkInterval = 20; // 1 second
const horizontalThreshold = 10;

// Particle configuration
const particleToSpawn = "multi:boss_beacon"; // Change as desired
const particleYOffset = 1.5; // How many blocks above the target coordinate to spawn particle

// --- Preprocessing: Generate unique tags ---
targetCoordinates.forEach(target => {
  if (!target.id) {
      console.error(`[Proximity Indicator] ERROR: Target coordinate missing 'id': ${JSON.stringify(target)}`);
      target.id = `${target.dimensionId}_${target.x}_${target.y}_${target.z}`.replace(/[:\-]/g, '_');
  }
  target.visitedTag = visitedTagPrefix + target.id;
});

console.log("[Proximity Indicator] Script Loaded (Unique Tags, Y/X Dist, Particle Cmd). Tags generated:");
targetCoordinates.forEach(t => console.log(` - ${t.name || t.id}: ${t.visitedTag}`));

// --- Script Logic ---
system.runInterval(() => {
  const players = world.getAllPlayers();

  for (const player of players) {
      const playerLocation = player.location;
      const playerDimension = player.dimension; // Store the dimension object

      let closestUnvisitedTarget = null;
      let minDistanceSquaredToUnvisited = Infinity;
      let verticalDifferenceToClosest = 0;
      let horizontalDifferenceXToClosest = 0;

      for (const target of targetCoordinates) {
          // Dimension Check
          if (playerDimension.id !== target.dimensionId) continue;
          // Visited Check (Handles stopping particle cmd too)
          if (player.hasTag(target.visitedTag)) continue;

          // Calculate Distances
          const dx = playerLocation.x - target.x;
          const dy = playerLocation.y - target.y;
          const dz = playerLocation.z - target.z;
          const distanceSquared = dx * dx + dy * dy + dz * dz;

          // Arrival Check
          if (distanceSquared <= arrivalRadiusSquared) {
              try {
                  player.addTag(target.visitedTag);
                  console.log(`[Proximity Indicator] Player ${player.name} arrived at '${target.name || target.id}'. Tagged with: ${target.visitedTag}`);
              } catch (error) {
                  console.error(`[Proximity Indicator] Failed to add tag '${target.visitedTag}' to player ${player.name}: ${error}`);
              }
              // Ensure this target isn't selected for action bar/particle this tick
              // by effectively removing it from consideration after tagging.
              // We use 'continue' which skips the proximity check below for *this* target.
              continue;
          }

          // Proximity Check
          if (distanceSquared <= proximityRadiusSquared) {
              if (distanceSquared < minDistanceSquaredToUnvisited) {
                  minDistanceSquaredToUnvisited = distanceSquared;
                  closestUnvisitedTarget = target;
                  verticalDifferenceToClosest = dy;
                  horizontalDifferenceXToClosest = dx;
              }
          }
      } // End loop through targets

      // --- Display Action Bar & Spawn Particle Command ---
      if (closestUnvisitedTarget) {
          // --- Action Bar ---
          try {
              const targetName = closestUnvisitedTarget.name || `Location [${Math.round(closestUnvisitedTarget.x)}, ...]`;
              const totalDistance = Math.round(Math.sqrt(minDistanceSquaredToUnvisited));

              // Calculate Vertical Indicator (Y-axis)
              const yDiff = Math.round(verticalDifferenceToClosest);
              let yIndicator = "";
              if (yDiff > 5) { yIndicator = ` §c↓${yDiff}m`; }
              else if (yDiff < -5) { yIndicator = ` §a↑${Math.abs(yDiff)}m`; }
              else if (yDiff !== 0) { yIndicator = ` §e↕${yDiff > 0 ? '+' : ''}${yDiff}m`; }
              else { yIndicator = " §b↕Level"; }

              // Calculate Horizontal Indicator (X-axis: East/West)
              const xDiff = Math.round(horizontalDifferenceXToClosest);
              let xIndicator = "";
              if (xDiff > horizontalThreshold) { xIndicator = ` §e W ${xDiff}m`; } // Target is West
              else if (xDiff < -horizontalThreshold) { xIndicator = ` §e E ${Math.abs(xDiff)}m`; } // Target is East

              // Combine Indicators
              player.onScreenDisplay.setActionBar(`§7Anomaly Detected: ${targetName} (§b${totalDistance}m${yIndicator}${xIndicator}§7)`);

          } catch (error) {
              console.error(`[Proximity Indicator] Failed to set action bar for player ${player.name}: ${error}`);
          }

          // --- Particle Command ---
          // Run a command to spawn particle AT the target location.
          // Note: Player will only see this if chunks are loaded & they are relatively close/looking that way.
          try {
              const { x, y, z } = closestUnvisitedTarget; // Get target coords
              const particleX = x.toFixed(2); // Use fixed precision for command
              const particleY = (y + particleYOffset).toFixed(2);
              const particleZ = z.toFixed(2);

              // Use the dimension object we stored earlier to run the command
              // runCommandAsync is preferred as it prevents script hangs if a command fails badly
               playerDimension.runCommandAsync(`particle ${particleToSpawn} ${particleX} ${particleY} ${particleZ}`);

          } catch (error) {
              // Log errors, potentially including which dimension/target failed
              console.error(`[Proximity Indicator] Failed to run particle command for target ${closestUnvisitedTarget.id} in dimension ${playerDimension.id}: ${error}`);
              if (error instanceof Error) {
                  console.error(error.stack); // Log stack trace for more details
              }
          }
      }
      // Optional: Clear action bar if no target found?

  } // End loop through players
}, checkInterval);






// --- Configuration ---
const RIFT_TITLE_MESSAGE = "--ERROR--";
// A unique identifier for the dynamic property to track if the dragon defeat event has fired.
const DRAGON_DEFEAT_EVENT_TRIGGERED_PROPERTY = "custom:dragon_defeat_title_triggered";
const ENDER_DRAGON_ENTITY_ID = "minecraft:ender_dragon";
// --- End Configuration ---


// Listen for any entity dying in the world
world.afterEvents.entityDie.subscribe(eventData => {
    const deadEntity = eventData.deadEntity;

    // Step 1: Check if the entity that died is the Ender Dragon
    if (deadEntity?.typeId === ENDER_DRAGON_ENTITY_ID) {
        console.log(`[Dragon Defeat Title] Detected Ender Dragon defeat.`); // Optional: Debug Log

        // Step 2: Check if the title event has already been triggered for this world
        const hasTriggered = world.getDynamicProperty(DRAGON_DEFEAT_EVENT_TRIGGERED_PROPERTY);

        // If it has already been triggered (is true), do nothing further.
        if (hasTriggered === true) {
            console.log("[Dragon Defeat Title] Dragon defeat title already shown in this world. Skipping."); // Optional: Debug Log
            return;
        }

        // If it hasn't triggered yet:
        console.log("[Dragon Defeat Title] First Ender Dragon defeat detected. Triggering title event."); // Optional: Debug Log

        // 3. Set the dynamic property to true so this doesn't run again on future dragon deaths.
        world.setDynamicProperty(DRAGON_DEFEAT_EVENT_TRIGGERED_PROPERTY, true);

        // 4. Schedule the title display for the next tick to ensure stability
        system.run(() => {
            try {
                // Get ALL players currently online, regardless of their dimension
                const players = world.getAllPlayers();

                // Loop through each online player
                for (const player of players) {
                    try {
                        // --- CORRECTED LINE ---
                        // Use the player's onScreenDisplay API with the correct method: setTitle
                        player.onScreenDisplay.setTitle(RIFT_TITLE_MESSAGE, {
                            // Optional parameters (uncomment/modify as needed):
                             stayDuration: 100, // Ticks (default 70)
                             fadeInDuration: 20, // Ticks (default 10)
                             fadeOutDuration: 20, // Ticks (default 20)
                            // subtitle: "The veil thins..." // Subtitle is set via options here too
                        });
                        // --- END CORRECTION ---
                        player.runCommandAsync(`execute as @s run scriptevent dialog:text °(Somewhere in Creation, a rip has been formed.;1;dialogue.txt2)`);
                        player.runCommandAsync(`give @s multi:key_forger`);
                    } catch (playerError) {
                        console.error(`[Dragon Defeat Title] Failed to send title to player ${player.name}: ${playerError}`);
                    }
                }
                if (players.length > 0) {
                    console.log(`[Dragon Defeat Title] Sent title to ${players.length} players.`);
                } else {
                    console.warn("[Dragon Defeat Title] Ender Dragon defeated, but no players were online to show the title to.");
                }
            } catch (error) {
                console.error(`[Dragon Defeat Title] Failed run title logic after dragon defeat: ${error}`);
            }
        });
    }
    // If the dead entity wasn't the dragon, the function just ends here.
});

// Optional: Log when the script loads for debugging
console.log("[Dragon Defeat Title] Script loaded successfully.");